@@ -5,13 +5,12 @@ use warnings;
 use Safe;
 use JSON::PP;
 use Dumpvalue;
-use CPAN::Version;
 use version ();
 use File::Spec ();
 use File::Temp ();
 use POSIX ':sys_wait_h';
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 our $VERBOSE = 0;
 our $ALLOW_DEV_VERSION = 0;
 
@@ -218,7 +217,7 @@ sub _packages_per_pmfile {
 
         $pline =~ s/\#.*//;
         next if $pline =~ /^\s*$/;
-        if ($pline =~ /\b__(?:END|DATA)__\b/
+        if ($pline =~ /^__(?:END|DATA)__\b/
             and $pmfile !~ /\.PL$/   # PL files may well have code after __DATA__
             ){
             last PLINE;
@@ -327,7 +326,7 @@ sub _packages_per_pmfile {
         while (<FH>) {
             $inpod = /^=(?!cut)/ ? 1 : /^=cut/ ? 0 : $inpod;
             next if $inpod || /^\s*#/;
-            last if /\b__(?:END|DATA)__\b/; # fails on quoted __END__ but this is rare
+            last if /^__(?:END|DATA)__\b/; # fails on quoted __END__ but this is rare -> __END__ in the middle of a line is rarer
             chop;
             # next unless /\$(([\w\:\']*)\bVERSION)\b.*\=/;
             next unless /([\$*])(([\w\:\']*)\bVERSION)\b.*\=/;
@@ -477,7 +476,7 @@ sub _normalize_version {
 # from PAUSE::pmfile;
 sub _force_numeric {
     my($self,$v) = @_;
-    $v = CPAN::Version->readable($v);
+    $v = $self->_readable($v);
 
     if (
         $v =~
@@ -532,6 +531,127 @@ sub _verbose {
     warn @what if $level <= $VERBOSE;
 }
 
+# all of the following methods are stripped from CPAN::Version
+# (as of version 5.5001, bundled in CPAN 2.03), and slightly
+# modified (ie. made private, as well as CPAN->debug(...) are
+# replaced with $self->_verbose(9, ...).)
+
+# CPAN::Version::vcmp courtesy Jost Krieger
+sub _vcmp {
+    my($self,$l,$r) = @_;
+    local($^W) = 0;
+    $self->_verbose(9, "l[$l] r[$r]");
+
+    return 0 if $l eq $r; # short circuit for quicker success
+
+    for ($l,$r) {
+        s/_//g;
+    }
+    $self->_verbose(9, "l[$l] r[$r]");
+    for ($l,$r) {
+        next unless tr/.// > 1 || /^v/;
+        s/^v?/v/;
+        1 while s/\.0+(\d)/.$1/; # remove leading zeroes per group
+    }
+    $self->_verbose(9, "l[$l] r[$r]");
+    if ($l=~/^v/ <=> $r=~/^v/) {
+        for ($l,$r) {
+            next if /^v/;
+            $_ = $self->_float2vv($_);
+        }
+    }
+    $self->_verbose(9, "l[$l] r[$r]");
+    my $lvstring = "v0";
+    my $rvstring = "v0";
+    if ($] >= 5.006
+     && $l =~ /^v/
+     && $r =~ /^v/) {
+        $lvstring = $self->_vstring($l);
+        $rvstring = $self->_vstring($r);
+        $self->_verbose(9, sprintf "lv[%vd] rv[%vd]", $lvstring, $rvstring);
+    }
+
+    return (
+            ($l ne "undef") <=> ($r ne "undef")
+            ||
+            $lvstring cmp $rvstring
+            ||
+            $l <=> $r
+            ||
+            $l cmp $r
+    );
+}
+
+sub _vgt {
+    my($self,$l,$r) = @_;
+    $self->_vcmp($l,$r) > 0;
+}
+
+sub _vlt {
+    my($self,$l,$r) = @_;
+    $self->_vcmp($l,$r) < 0;
+}
+
+sub _vge {
+    my($self,$l,$r) = @_;
+    $self->_vcmp($l,$r) >= 0;
+}
+
+sub _vle {
+    my($self,$l,$r) = @_;
+    $self->_vcmp($l,$r) <= 0;
+}
+
+sub _vstring {
+    my($self,$n) = @_;
+    $n =~ s/^v// or die "Parse::PMFile::_vstring() called with invalid arg [$n]";
+    pack "U*", split /\./, $n;
+}
+
+# vv => visible vstring
+sub _float2vv {
+    my($self,$n) = @_;
+    my($rev) = int($n);
+    $rev ||= 0;
+    my($mantissa) = $n =~ /\.(\d{1,12})/; # limit to 12 digits to limit
+                                          # architecture influence
+    $mantissa ||= 0;
+    $mantissa .= "0" while length($mantissa)%3;
+    my $ret = "v" . $rev;
+    while ($mantissa) {
+        $mantissa =~ s/(\d{1,3})// or
+            die "Panic: length>0 but not a digit? mantissa[$mantissa]";
+        $ret .= ".".int($1);
+    }
+    # warn "n[$n]ret[$ret]";
+    $ret =~ s/(\.0)+/.0/; # v1.0.0 => v1.0
+    $ret;
+}
+
+sub _readable {
+    my($self,$n) = @_;
+    $n =~ /^([\w\-\+\.]+)/;
+
+    return $1 if defined $1 && length($1)>0;
+    # if the first user reaches version v43, he will be treated as "+".
+    # We'll have to decide about a new rule here then, depending on what
+    # will be the prevailing versioning behavior then.
+
+    if ($] < 5.006) { # or whenever v-strings were introduced
+        # we get them wrong anyway, whatever we do, because 5.005 will
+        # have already interpreted 0.2.4 to be "0.24". So even if he
+        # indexer sends us something like "v0.2.4" we compare wrongly.
+
+        # And if they say v1.2, then the old perl takes it as "v12"
+
+        $self->_verbose(9, "Suspicious version string seen [$n]\n");
+        return $n;
+    }
+    my $better = sprintf "v%vd", $n;
+    $self->_verbose(9, "n[$n] better[$better]");
+    return $better;
+}
+
 1;
 
 __END__
@@ -575,10 +695,12 @@ Set this to true if you need to know some details.
 
 =head1 SEE ALSO
 
-Most part of this module is derived from PAUSE.
+Most part of this module is derived from PAUSE and CPAN::Version.
 
 L<https://github.com/andk/pause>
 
+L<https://github.com/andk/cpanpm>
+
 =head1 AUTHOR
 
 Andreas Koenig E<lt>andreas.koenig@anima.deE<gt>
